#!/bin/bash

# ------------------------------------------------------------------
# The follwing is section reporduces a problem with how cargo deals 
# with the same crate exposed at different locations.
#
# Bug manifests in cargo 1.45.1 (f242df6ed 2020-07-22)
# Error message displayed:
#   
#   error: package collision in the lockfile: packages 
#   d v0.1.0 (/tmp/tmp.TfZoQkYrMp/a/b/d) and 
#   d v0.1.0 (/tmp/tmp.TfZoQkYrMp/a/c/d) are different, 
#   but only one can be written to lockfile unambiguously
#
# ------------------------------------------------------------------


TMPDIR="$(mktemp -d)"
cd $TMPDIR
cargo new --lib a
cargo new --lib b
cargo new --lib c
cargo new --lib d
# Move crate d into crates b and c and add as a dependency to both
cp -a d b
mv    d c
echo "d = { path = \"d\" }" >> b/Cargo.toml 
echo "d = { path = \"d\" }" >> c/Cargo.toml 
# Move b and c into crate a and add both as a dependency of a
mv b a
mv c a
echo "b = { path = \"b\" }" >> a/Cargo.toml 
echo "c = { path = \"c\" }" >> a/Cargo.toml 
# change cwd to a and compile
cd a

echo ""
echo "Cargo will now fail:"
cargo test 2>&1 | cat



# ------------------------------------------------------------------
# Byond this point is not directly relavant to reproducing this bug 
# and may be ignored or discarded. 
#
# The following demonstrates that this issue is not caused by rustc.
# ------------------------------------------------------------------

set -euH
trap "[[ -d '$TMPDIR' ]] && echo 'cleaning up...' && rm -r '$TMPDIR'" EXIT INT TERM

# Same content in d
echo 'pub fn d() { println!("hello from d.") }' > b/d/src/lib.rs
echo 'pub fn d() { println!("hello from d.") }' > c/d/src/lib.rs

echo 'extern crate d; pub fn b() { d::d() }' > b/src/lib.rs
echo 'extern crate d; pub fn c() { d::d() }' > c/src/lib.rs

echo 'extern crate b; extern crate c; fn main() { b::b(); c::c(); }' > src/lib.rs

# set -x 

# Build b and b's d.
cd b/d
rustc --crate-type=rlib --crate-name=d "src/lib.rs"
cd ..
rustc --crate-type=rlib --crate-name=b "src/lib.rs" -L d
cd ..

# Build c and c's d.
cd c/d
rustc --crate-type=rlib --crate-name=d "src/lib.rs"
cd ..
rustc --crate-type=rlib --crate-name=c "src/lib.rs" -L d
cd ..

# Build a
echo ""
echo "Rustc will not fail if the contents of both ds are identical.."
rustc --crate-type=bin --crate-name=a "src/lib.rs" -L b -L c -L b/d  -L c/d

# Run a
./a 

# different content in each d
echo 'pub fn d() { println!("hello from bd.") }' > b/d/src/lib.rs
echo 'pub fn d() { println!("hello from cd.") }' > c/d/src/lib.rs

# Build b and b's d.
cd b/d
rustc --crate-type=rlib --crate-name=d "src/lib.rs"
cd ..
rustc --crate-type=rlib --crate-name=b "src/lib.rs" -L d
cd ..

# Build c and c's d.
cd c/d
rustc --crate-type=rlib --crate-name=d "src/lib.rs"
cd ..
rustc --crate-type=rlib --crate-name=c "src/lib.rs" -L d
cd ..

# Build a
echo ""
echo "Rustc will fail if the contents of both ds differ:"
rustc --crate-type=bin --crate-name=a "src/lib.rs" -L b -L c -L b/d  -L c/d 2>&1 | cat








